﻿/********************************************************************************/
/* 	keypad.d																	*/
/*------------------------------------------------------------------------------*/
/*	製作		（　゜ワ゜）ノ　／　松久貫太									*/
/*	製作開始　	2007/05/11														*/
/*	MAIL		omega@personal.email.ne.jp										*/
/*	URL		http://nagoya.cool.ne.jp/o_mega										*/
/*																				*/
/*	このソースは「やわらかライセンス」の元で配布されています。					*/
/*-更新履歴---------------------------------------------------------------------*/
/*	2007/--/--																	*/
/*-その他-----------------------------------------------------------------------*/
/*	リプレイ機能の追加(D.K)														*/
/********************************************************************************/

/* ライブラリのインポート *******************************************************/
private import hell2;		// dHell2ライブラリ
private import std.file;
private import std.stream;
private import std.stdio;


class KeypadState{
	bool PressEnter;
	bool PressCancel;
	bool PressLeft;
	bool PressUp;
	bool PressRight;
	bool PressDown;
	
	bool PushEnter;
	bool PushCancel;
	bool PushLeft;
	bool PushUp;
	bool PushRight;
	bool PushDown;
	
	this(){
		PressEnter = false;
		PressCancel = false;
		PressLeft = false;
		PressUp = false;
		PressRight = false;
		PressDown = false;
		
		PushEnter = false;
		PushCancel = false;
		PushLeft = false;
		PushUp = false;
		PushRight = false;
		PushDown = false;
	}
}

class Keypad
{
	enum{
		RECORD,
		REPLAY,
	}

	const static int RECORDABLE_MAX = 60 * 60 * 10;	// 10分
	
	KeypadState[] ks = null;
	int ptr = 0;
	int length = 0;
	int mode;

	bool UseEnter;
	bool UseCancel;
	bool UseLeft;
	bool UseUp;
	bool UseRight;
	bool UseDown;
	
	int player = -1;
	this(int player = 0)
	{
		this.player = player;

		this.ks = new KeypadState[RECORDABLE_MAX];
		foreach(inout k;ks){k = new KeypadState();}
		this.ptr = 0;
		this.length = 0;

		UseEnter = true;
		UseCancel = true;
		UseLeft = true;
		UseUp = true;
		UseRight = true;
		UseDown = true;
		
		this.mode = RECORD;
	}
	

	void update(){
		ptr++;
		
		if( ptr >= RECORDABLE_MAX )
		{
			ptr = RECORDABLE_MAX - 1;	//とりあえす落ちないようにしとく
		}
		
		if(mode == RECORD){
			if(ptr >= length) length = ptr + 1;
			
			ks[ptr].PressEnter = ( isPressEnter() & UseEnter );
			ks[ptr].PressCancel = ( isPressCancel() & UseCancel );
			ks[ptr].PressLeft = ( isPressLeft() & UseLeft );
			ks[ptr].PressUp = ( isPressUp() & UseUp );
			ks[ptr].PressRight = ( isPressRight() & UseRight );
			ks[ptr].PressDown = ( isPressDown() & UseDown );
		
			ks[ptr].PushEnter = ( isPushEnter() & UseEnter );
			ks[ptr].PushCancel = ( isPushCancel() & UseCancel );
			ks[ptr].PushLeft = ( isPushLeft() & UseLeft );
			ks[ptr].PushUp = ( isPushUp() & UseUp );
			ks[ptr].PushRight = ( isPushRight() & UseRight );
			ks[ptr].PushDown = ( isPushDown() & UseDown );

		}else if(mode == REPLAY){
			if(ptr >= length) ptr = length - 1;
		}
	}
	//-----
	bool save(string filename){
		auto fp = new BufferedFile();
		fp.create(filename);
		
		for(int ptr=0;ptr<=length;ptr++){
			fp.write(cast(short)ks[ptr].PressEnter);
			fp.write(cast(short)ks[ptr].PressCancel);
			fp.write(cast(short)ks[ptr].PressLeft);
			fp.write(cast(short)ks[ptr].PressUp);
			fp.write(cast(short)ks[ptr].PressRight);
			fp.write(cast(short)ks[ptr].PressDown);
		
			fp.write(cast(short)ks[ptr].PushEnter);
			fp.write(cast(short)ks[ptr].PushCancel);
			fp.write(cast(short)ks[ptr].PushLeft);
			fp.write(cast(short)ks[ptr].PushUp);
			fp.write(cast(short)ks[ptr].PushRight);
			fp.write(cast(short)ks[ptr].PushDown);
		}
		fp.close();
		return true;
	}
	
	bool load(string filename){
		bool rc = true;
		int ptr=0;
		auto fp = new BufferedFile(filename);
		
		try{
			while(!fp.eof()){
				short buf;
				
				fp.read(buf);
				ks[ptr].PressEnter = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PressCancel = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PressLeft = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PressUp = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PressRight = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PressDown = cast(bool)buf;

				fp.read(buf);
				ks[ptr].PushEnter = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PushCancel = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PushLeft = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PushUp = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PushRight = cast(bool)buf;
				fp.read(buf);
				ks[ptr].PushDown = cast(bool)buf;

				ptr++;
			}
		}catch(Exception e){
			Hell_write(e);
			rc = false;
		}
		
		length = ptr;
		mode = REPLAY;
		
		return ( rc );
	}
	/*
	 * リプレイのキーの有効、無効を設定　無効の場合は保存されない
	 * 引数   : true:有効、false:無効
	 * 戻り値 : なし
	 */
	void SetUseEnter( bool use )
	{
		UseEnter = use;
	}
	void SetUseCancel( bool use )
	{
		UseCancel = use;
	}
	void SetUseLeft( bool use )
	{
		UseLeft = use;
	}
	void SetUseUp( bool use )
	{
		UseUp = use;
	}
	void SetUseRight( bool use )
	{
		UseRight = use;
	}
	void SetUseDown( bool use )
	{
		UseDown = use;
	}


	/*
	 * エンターキーを押している(KeyPress)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 押しているならtrue
	 */
	bool isPressEnter( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPressKey(HELL_z) && player == 0) return true;
			if(Hell_isPressKey(HELL_SPACE) && player == 0) return true;
			if(Hell_isPressJButton(0 , player))  return true;
		}
		else 
		{
			if(ks[ptr].PressEnter && player == 0) return true;
		}
		return false;
	}

	/*
	 * エンターキーを押した(KeyDown)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 押したならtrue
	 */
	bool isPushEnter( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPushKey(HELL_z) && player == 0) return true;
			if(Hell_isPushKey(HELL_SPACE) && player == 0) return true;
			if(Hell_isPushJButton(0 , player))  return true;
		}
		else 
		{
			if(ks[ptr].PushEnter && player == 0) return true;
		}
		return false;
	}

	/*
	 * キャンセルキーを押しているか(KeyPress)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 押しているならtrue
	 */
	bool isPressCancel( bool getsyskey = false )
	{	
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPressKey(HELL_x) && player == 0) return true;
			if(Hell_isPressKey(HELL_RETURN) && player == 0) return true;
			if(Hell_isPressJButton(1 , player))  return true;
		}
		else 
		{
			if(ks[ptr].PressCancel && player == 0) return true;
		}
		return false;
	}
	
	/*
	 * キャンセルキーを押しているか(KeyDown)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 押しているならtrue
	 */
	bool isPushCancel( bool getsyskey = false )
	{	
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPushKey(HELL_x) && player == 0) return true;
			if(Hell_isPushKey(HELL_RETURN) && player == 0) return true;
			if(Hell_isPushJButton(1 , player))  return true;
		}
		else 
		{
			if(ks[ptr].PushCancel && player == 0) return true;
		}
		return false;
	}

	/*
	 * 左キーを押している(KeyPress)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 左キーを押しているならtrue
	 */
	bool isPressLeft( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPressKey(HELL_LEFT) && player == 0)    return true;
			if(Hell_isPressJKey(HELL_J_LEFT , player)) return true;
		}
		else 
		{
			if(ks[ptr].PressLeft && player == 0) return true;
		}
		return false;
	}
	/*
	 * 上キーを押している(KeyPress)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 上キーを押しているならtrue
	 */
	bool isPressUp( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPressKey(HELL_UP) && player == 0)    return true;
			if(Hell_isPressJKey(HELL_J_UP , player)) return true;
		}
		else 
		{
			if(ks[ptr].PressUp && player == 0) return true;
		}
		return false;
	}
	/*
	 * 右キーを押している(KeyPress)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 右キーを押しているならtrue
	 */
	bool isPressRight( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPressKey(HELL_RIGHT) && player == 0)    return true;
			if(Hell_isPressJKey(HELL_J_RIGHT , player)) return true;
		}
		else 
		{
			if(ks[ptr].PressRight && player == 0) return true;
		}
		return false;
	}
	/*
	 * 下キーを押している(KeyPress)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 下キーを押しているならtrue
	 */
	bool isPressDown( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPressKey(HELL_DOWN) && player == 0)    return true;
			if(Hell_isPressJKey(HELL_J_DOWN , player)) return true;
		}
		else 
		{
			if(ks[ptr].PressDown && player == 0) return true;
		}
		return false;
	}

	// Down

	/*
	 * 左キーを押した(KeyDown)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 左キーを押したならtrue
	 */
	bool isPushLeft( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPushKey(HELL_LEFT) && player == 0)    return true;
			if(Hell_isPushJKey(HELL_J_LEFT , player)) return true;
		}
		else 
		{
			if(ks[ptr].PushLeft && player == 0) return true;
		}
		return false;
	}
	/*
	 * 上キーを押した(KeyDown)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 上キーを押したならtrue
	 */
	bool isPushUp( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPushKey(HELL_UP) && player == 0)    return true;
			if(Hell_isPushJKey(HELL_J_UP , player)) return true;
		}
		else 
		{
			if(ks[ptr].PushUp && player == 0) return true;
		}
		return false;
	}
	/*
	 * 右キーを押した(KeyDown)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 右キーを押したならtrue
	 */
	bool isPushRight( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPushKey(HELL_RIGHT) && player == 0)    return true;
			if(Hell_isPushJKey(HELL_J_RIGHT , player)) return true;
		}
		else 
		{
			if(ks[ptr].PushRight && player == 0) return true;
		}
		return false;
	}
	/*
	 * 下キーを押した(KeyDown)か
	 * 引数   : getsyskey　リプレイの有無に関わらずキーを取得するか
	 * 戻り値 : 下キーを押したならtrue
	 */
	bool isPushDown( bool getsyskey = false )
	{
		if ( ( mode == RECORD ) || ( getsyskey == true ) )
		{
			if(Hell_isPushKey(HELL_DOWN) && player == 0)    return true;
			if(Hell_isPushJKey(HELL_J_DOWN , player)) return true;
		}
		else 
		{
			if(ks[ptr].PushDown && player == 0) return true;
		}
		return false;
	}
}

//--------------------------------------------------------------------

class MouseState{
	int button;
	int cursor_x;
	int cursor_y;
	
	this(){
		button = cursor_x = cursor_y = 0;
	}
}

class MouseRecorder{
	enum{
		RECORD,
		REPLAY,
	}
	
	const static int RECORDABLE_MAX = 60 * 60 * 10;	// 10分
	
	MouseState[] ms = null;
	int ptr = 0;
	int length = 0;
	int mode;
	
	this(){
		ms = new MouseState[RECORDABLE_MAX];
		foreach(inout m;ms){m = new MouseState();}
		ptr = 0;
		length = 0;
		
		mode = RECORD;
	}
	
	void update(){
		ptr++;
		
		if(mode == RECORD){
			if(ptr >= length) length = ptr + 1;
			
			ms[ptr].cursor_x = Hell_getMouseX();
			ms[ptr].cursor_y = Hell_getMouseY();
			ms[ptr].button = Hell_isPressMouse();
		}else if(mode == REPLAY){
			if(ptr >= length) ptr = length - 1;
		}
	}
	
	void setMode(int mode){
		this.mode = mode;
		this.ptr = 0;
	}
	int getMode(){return mode;}
	
	int getX(){
		if(ptr >= length || ptr < 0) return 0;
		return ms[ptr].cursor_x;
	}
	int getY(){
		if(ptr >= length || ptr < 0) return 0;
		return ms[ptr].cursor_y;
	}
	int isPress(){
		if(ptr >= length || ptr < 0) return 0;
		return ms[ptr].button;
	}
	int isPush(){
		if(ptr >= length || ptr < 1) return 0;
		return ms[ptr].button & (!ms[ptr - 1].button);
	}
	
	//-----
	bool save(string filename){
		auto fp = new BufferedFile();
		fp.create(filename);
		
		for(int ptr=0;ptr<=length;ptr++){
			fp.write(cast(short)ms[ptr].cursor_x);
			fp.write(cast(short)ms[ptr].cursor_y);
			fp.write(cast(short)ms[ptr].button);
		}
		fp.close();
		return true;
	}
	
	bool load(string filename){
		int ptr=0;
		auto fp = new BufferedFile(filename);
		
		try{
			while(!fp.eof()){
				short buf;
				
				fp.read(buf);
				ms[ptr].cursor_x = cast(int)buf;
				
				fp.read(buf);
				ms[ptr].cursor_y = cast(int)buf;
				
				fp.read(buf);
				ms[ptr].button = cast(int)buf;
				ptr++;
			}
		}catch(Exception e){
			Hell_write(e);
		}
		
		length = ptr;
		mode = REPLAY;
		
		return false;
	}
}